/*
 * cache_test.c
 * Simple program to test the execution time of the Sitsang when the task
 * does heavy memory accesses
 *
 *  3/15/04 LCS -- Created
 *  3/29/04 LCS -- Added DWCS configuration code
 *  6/25/04 LCS -- Stripped out everything except for array accesses.
 *                 DWCS params/monitoring is now handled by dwcs_execute
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include "sched.h"
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "dwcs_sched.h"
#include <sys/wait.h>

#define MISS_ARRAY_SIZE 16384

inline void mem_access(int *p)
{
    *p = (*p + 1000000) / 2;
}

int main(int argc, char **argv)
{
    int miss_array[MISS_ARRAY_SIZE + 1];
    int *array_pos;
    int n, max, offset;
    int miss_rate;

    if (argc != 3)
    {
        fprintf(stderr, "Usage: cache_test <iterations> <miss_rate*1024>\n");
        return -1;
    }
    max = atoi(argv[1]);
    miss_rate = atoi(argv[2]);


    /*
     * Pre-access every page to fill the TLB
     */
    for (n=0; n<(MISS_ARRAY_SIZE+1); n += 1024)
    {
        array_pos = miss_array + n;
        *array_pos++;
    }

    /*
     * Each individual array element should converge to 1,000,000
     * The PXA255 only has 32KB of data cache, so if we use a really big
     * array, we should cause lots of cache misses
     */
    offset = 0;
    for (n=0; n<max; n++)
    {
        if ((n & 1023) < miss_rate)
        {
            // miss
            mem_access(miss_array + offset);
            offset = (offset + 8) & (MISS_ARRAY_SIZE - 1);
        }
        else
        {
            // hit
            mem_access(miss_array + MISS_ARRAY_SIZE);
        }
    }

    return 0;
}


