/*
 * dwcs_execute.c
 * Runs any command as a DWCS process and prints statistics.
 *
 *  6/23/04 LCS -- Created
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include "sched.h"
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "dwcs_sched.h"
#include <sys/wait.h>


#define WDONTREAP 0x00000004

inline void display_dwcs_info(void)
{
    execlp("cat", "cat", "/proc/dwcs/info", NULL);
    exit(0);
}

int main(int argc, char **argv)
{
    struct timeval tv_begin, tv_end;
    int n, time;
    struct dwcs_sched_param sp;
    int ret;
    int period;
    int x;
    int y;
    pid_t pid;
    char **new_argv;

    if (argc < 6)
    {
        fprintf(stderr, "Usage: dwcs_execute <period> <x> <y> <time> <program> [<arg1> [<arg2> [...]]]\n");
        return -1;
    }
    period = atoi (argv[1]);
    x = atoi (argv[2]);
    y = atoi (argv[3]);
    time = atoi (argv[4]);

    new_argv = malloc(sizeof(char*) * (argc - 6 + 2));
    assert(new_argv != NULL);
    for (n=0; n<argc - 6 + 1; n++)
        new_argv[n] = argv[n + 5];
    new_argv[argc - 6 + 1] = NULL;

    gettimeofday(&tv_begin, NULL);

    pid = fork();
    if (pid == 0)
    {
        execvp(argv[5], new_argv);
        exit(0);
    }

    sp.pid = pid;
    sp.policy = SCHED_DWCS;
    sp.period = period;
    sp.own = x;
    sp.owd = y;
    sp.flags = 0;
    sp.service_time = time;

    printf("Open /proc\n");
    if ((ret = open ("/proc/dwcs/control", 0)) < 0) {
        printf ("error opening control\n");
    }

    printf("Use ioctl\n");
    if (ioctl (ret, DWCSCTL_SCHED_SETPARAM, &sp, (char *)0) < 0) {
        printf ("ioctl error\n");
    }

    /* Wait for process to complete */
    waitpid(pid, NULL, WDONTREAP);

    gettimeofday(&tv_end, NULL);

    time = ((tv_end.tv_sec - tv_begin.tv_sec) * 1000) + ((tv_end.tv_usec -
        tv_begin.tv_usec) / 1000);
    printf("Task finished in %d ms.\n", time);
    fflush(stdout);

    // display dwcs info
    display_dwcs_info();

    return 0;
}


